package java.net;

import sun.misc.SharedSecrets;
import java.io.IOException;
import sun.misc.JavaIOFileDescriptorAccess;

class DualStackPlainDatagramSocketImpl extends AbstractPlainDatagramSocketImpl
{
    static JavaIOFileDescriptorAccess fdAccess;
    private final boolean exclusiveBind;
    private boolean reuseAddressEmulated;
    private boolean isReuseAddress;
    
    DualStackPlainDatagramSocketImpl(boolean exclusiveBind) {
        this.exclusiveBind = exclusiveBind;
    }
    
    @Override
    protected void datagramSocketCreate() throws SocketException {
        if (this.fd == null) {
            throw new SocketException("Socket closed");
        }
        DualStackPlainDatagramSocketImpl.fdAccess.set(this.fd, socketCreate(false));
        return;
    }
    
    @Override
    protected synchronized void bind0(int localPort, InetAddress inetAddress) throws SocketException {
        int checkAndReturnNativeFD = this.checkAndReturnNativeFD();
        if (inetAddress == null) {
            throw new NullPointerException("argument address");
        }
        socketBind(checkAndReturnNativeFD, inetAddress, localPort, this.exclusiveBind);
        if (localPort == 0) {
            this.localPort = socketLocalPort(checkAndReturnNativeFD);
        }
        else {
            this.localPort = localPort;
        }
        return;
    }
    
    @Override
    protected synchronized int peek(InetAddress address) throws IOException {
        this.checkAndReturnNativeFD();
        if (address == null) {
            throw new NullPointerException("Null address in peek()");
        }
        DatagramPacket datagramPacket = new DatagramPacket(new byte[1], 1);
        int peekData = this.peekData(datagramPacket);
        address = datagramPacket.getAddress();
        return peekData;
    }
    
    @Override
    protected synchronized int peekData(DatagramPacket datagramPacket) throws IOException {
        int checkAndReturnNativeFD = this.checkAndReturnNativeFD();
        if (datagramPacket == null) {
            throw new NullPointerException("packet");
        }
        if (datagramPacket.getData() == null) {
            throw new NullPointerException("packet buffer");
        }
        return socketReceiveOrPeekData(checkAndReturnNativeFD, datagramPacket, this.timeout, this.connected, true);
    }
    
    @Override
    protected synchronized void receive0(DatagramPacket datagramPacket) throws IOException {
        int checkAndReturnNativeFD = this.checkAndReturnNativeFD();
        if (datagramPacket == null) {
            throw new NullPointerException("packet");
        }
        if (datagramPacket.getData() == null) {
            throw new NullPointerException("packet buffer");
        }
        socketReceiveOrPeekData(checkAndReturnNativeFD, datagramPacket, this.timeout, this.connected, false);
        return;
    }
    
    @Override
    protected void send(DatagramPacket datagramPacket) throws IOException {
        int checkAndReturnNativeFD = this.checkAndReturnNativeFD();
        if (datagramPacket == null) {
            throw new NullPointerException("null packet");
        }
        if (datagramPacket.getAddress() == null || datagramPacket.getData() == null) {
            throw new NullPointerException("null address || null buffer");
        }
        socketSend(checkAndReturnNativeFD, datagramPacket.getData(), datagramPacket.getOffset(), datagramPacket.getLength(), datagramPacket.getAddress(), datagramPacket.getPort(), this.connected);
        return;
    }
    
    @Override
    protected void connect0(InetAddress inetAddress, int n) throws SocketException {
        int checkAndReturnNativeFD = this.checkAndReturnNativeFD();
        if (inetAddress == null) {
            throw new NullPointerException("address");
        }
        socketConnect(checkAndReturnNativeFD, inetAddress, n);
        return;
    }
    
    @Override
    protected void disconnect0(int n) {
        if (this.fd == null || !this.fd.valid()) {
            return;
        }
        socketDisconnect(DualStackPlainDatagramSocketImpl.fdAccess.get(this.fd));
        return;
    }
    
    @Override
    protected void datagramSocketClose() {
        if (this.fd == null || !this.fd.valid()) {
            return;
        }
        socketClose(DualStackPlainDatagramSocketImpl.fdAccess.get(this.fd));
        DualStackPlainDatagramSocketImpl.fdAccess.set(this.fd, -1);
        return;
    }
    
    @Override
    protected void socketSetOption(int n, Object o) throws SocketException {
        int checkAndReturnNativeFD = this.checkAndReturnNativeFD();
        int intValue;
        switch (n) {
            case 3:
            case 4097:
            case 4098: {
                intValue = (int)o;
                break;
            }
            case 4: {
                if (this.exclusiveBind && this.localPort != 0) {
                    this.reuseAddressEmulated = true;
                    this.isReuseAddress = (boolean)o;
                    return;
                }
            }
            case 32: {
                intValue = (((boolean)o) ? 1 : 0);
                break;
            }
            default: {
                throw new SocketException("Option not supported");
            }
        }
        socketSetIntOption(checkAndReturnNativeFD, n, intValue);
    }
    
    @Override
    protected Object socketGetOption(int n) throws SocketException {
        int checkAndReturnNativeFD = this.checkAndReturnNativeFD();
        if (n == 15) {
            return socketLocalAddress(checkAndReturnNativeFD);
        }
        if (n == 4 && this.reuseAddressEmulated) {
            return Boolean.valueOf(this.isReuseAddress);
        }
        int socketGetIntOption = socketGetIntOption(checkAndReturnNativeFD, n);
        Boolean b;
        switch (n) {
            case 4:
            case 32: {
                b = ((socketGetIntOption == 0) ? Boolean.FALSE : Boolean.TRUE);
                break;
            }
            case 3:
            case 4097:
            case 4098: {
                b = new Integer(socketGetIntOption);
                break;
            }
            default: {
                throw new SocketException("Option not supported");
            }
        }
        return b;
    }
    
    @Override
    protected void join(InetAddress inetAddress, NetworkInterface networkInterface) throws IOException {
        throw new IOException("Method not implemented!");
    }
    
    @Override
    protected void leave(InetAddress inetAddress, NetworkInterface networkInterface) throws IOException {
        throw new IOException("Method not implemented!");
    }
    
    @Override
    protected void setTimeToLive(int n) throws IOException {
        throw new IOException("Method not implemented!");
    }
    
    @Override
    protected int getTimeToLive() throws IOException {
        throw new IOException("Method not implemented!");
    }
    
    @Deprecated
    @Override
    protected void setTTL(byte b) throws IOException {
        throw new IOException("Method not implemented!");
    }
    
    @Deprecated
    @Override
    protected byte getTTL() throws IOException {
        throw new IOException("Method not implemented!");
    }
    
    private int checkAndReturnNativeFD() throws SocketException {
        if (this.fd == null || !this.fd.valid()) {
            throw new SocketException("Socket closed");
        }
        return DualStackPlainDatagramSocketImpl.fdAccess.get(this.fd);
    }
    
    private static native void initIDs();
    
    private static native int socketCreate(boolean p0);
    
    private static native void socketBind(int p0, InetAddress p1, int p2, boolean p3) throws SocketException;
    
    private static native void socketConnect(int p0, InetAddress p1, int p2) throws SocketException;
    
    private static native void socketDisconnect(int p0);
    
    private static native void socketClose(int p0);
    
    private static native int socketLocalPort(int p0) throws SocketException;
    
    private static native Object socketLocalAddress(int p0) throws SocketException;
    
    private static native int socketReceiveOrPeekData(int p0, DatagramPacket p1, int p2, boolean p3, boolean p4) throws IOException;
    
    private static native void socketSend(int p0, byte[] p1, int p2, int p3, InetAddress p4, int p5, boolean p6) throws IOException;
    
    private static native void socketSetIntOption(int p0, int p1, int p2) throws SocketException;
    
    private static native int socketGetIntOption(int p0, int p1) throws SocketException;
    
    @Override
    native int dataAvailable();
    
    static {
        DualStackPlainDatagramSocketImpl.fdAccess = SharedSecrets.getJavaIOFileDescriptorAccess();
        initIDs();
    }
}
